/* 
 * Author: Stian Mikalsen
 * Copyright: Stian Mikalsen, 2023
 * License: Gnu GPL V2

ATtiny85
                            ^____
                        pb5 |    | Vcc                          
  out: ecu          <-- pb3 |    | pb2 --> in, from sensor.
  out: pwm brak led <-- pb4 |    | pb1 -->
                        Gnd |____| pb0 --> 
 
Description!.
Dynamic break light.

1* If the wheel is doing a hard deceleration.
As in, a high change in the delta then we want to rapid flash the break light for 1sec.
A pwm signal on PB4 is setup to drive the led, the flashes happens between half and full strength (0x80-0xff).

2 * If the wheel is doing a soft/slow deceleration. 
The pwm signal should slowly increase until it reaches 0xff.

3 * If the wheel is having a constant speed or accelerating.
The pwm signal should slowly decrease until it reaches 0x80.


Pulses generated by the wheel are 16, 8high and 8high per revolution.
The ISR(INT0_vect) increments the pulses done as the wheel rotates and generates interrupts. 

ISR(TIMER0_COMPA_vect) is setup as a 10Hz timer.
Every 10th time it triggers,(1sec) we do a comparison.
Compare the current pulses done with number of pulses done in the previous 1second.

 */
//#define F_CPU 1000000UL 



#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include "macros.h"

#include "avr_mcu_section.h"
AVR_MCU(F_CPU, "attiny85");

const struct avr_mmcu_vcd_trace_t _mytrace[]  _MMCU_ = {
	{ AVR_MCU_VCD_SYMBOL("PORTB"), .what = (void*)&PORTB, }
};

volatile char tcount = 0;
volatile uint8_t pulse_rotation = 0;
volatile uint8_t pulse_count = 0;
volatile uint8_t pulse_count_old = 0;
volatile int8_t delta = 0;

int main(void) {
    
    cli();

    // -------------------------------------------------------
    BITS_CLEAR(DDRB, PB2);  // Input from sensor


    // Output.
    BITS_SET(DDRB, PB3);  
    BITS_CLEAR(PORTB, PB3); // Set the state low.	

    // Output.
    BITS_SET(DDRB, PB4);  
    BITS_CLEAR(PORTB, PB4); // Set the state low.

    // -------------------------------
    // Setup Timer0,
    // the timer is setup for ~ 10hz.
    TCCR0A |= (1 << WGM01); 
    TCCR0B |= (1 << CS02) | (1 << CS00); 
    OCR0A = 97;

    // -------------------------------
    // Set Timer/Counter1 in Fast PWM mode with a prescaler of 8
 	GTCCR |= (1<<PWM1B) | (1<<COM1B0) | (1<<COM1B1);
    TCCR1 |= (1 << COM1A0) | (1 << CS10); 
    OCR1B = 0x32; 

    // -------------------------------
    // Interrupt stuff 
    //MCUCR |= (1 << ISC01) | (1  << ISC00);	// The rising edge of INT0 generates an interrupt request.
    MCUCR |= (1 << ISC00);						// set INT0 to trigger on any logical change.

    // -------------------------------
    // enable interrupt stuff
    GIMSK |= (1 << INT0);
    TIMSK |= (1 << OCIE0A);

    sei();

    while (1) {}
    return 0;
}

ISR(TIMER0_COMPA_vect) {
     tcount++;
    if (tcount >= 10) {
        tcount = 0;
        pulse_count = 0;
    } 
}

ISR(INT0_vect) {
    pulse_count++;
    pulse_rotation++;
    if (pulse_rotation == 16) {
        pulse_rotation = 0;
        BITS_FLIP(PORTB, PB3);
    }
    //BITS_FLIP(PORTB, PB4);
}